'use strict';

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var React = _interopDefault(require('react'));
var createStore = _interopDefault(require('picostate'));
var matchit = require('matchit');
var react = require('@picostate/react');

function matcher(routes) {
  var matchers = routes.map(function (route) {
    return matchit.parse(route[0]);
  });
  var dictionary = routes.reduce(function (dict, route) {
    dict[route[0]] = route[1];
    return dict;
  }, {});
  return function route(path) {
    var m = matchit.match(path, matchers);
    if (!m.length) return [];
    return [dictionary[m[0].old], matchit.exec(path, m)];
  };
}

function _classCallCheck(instance, Constructor) {
  if (!(instance instanceof Constructor)) {
    throw new TypeError("Cannot call a class as a function");
  }
}

function _defineProperties(target, props) {
  for (var i = 0; i < props.length; i++) {
    var descriptor = props[i];
    descriptor.enumerable = descriptor.enumerable || false;
    descriptor.configurable = true;
    if ("value" in descriptor) descriptor.writable = true;
    Object.defineProperty(target, descriptor.key, descriptor);
  }
}

function _createClass(Constructor, protoProps, staticProps) {
  if (protoProps) _defineProperties(Constructor.prototype, protoProps);
  if (staticProps) _defineProperties(Constructor, staticProps);
  return Constructor;
}

function _inherits(subClass, superClass) {
  if (typeof superClass !== "function" && superClass !== null) {
    throw new TypeError("Super expression must either be null or a function");
  }

  subClass.prototype = Object.create(superClass && superClass.prototype, {
    constructor: {
      value: subClass,
      writable: true,
      configurable: true
    }
  });
  if (superClass) _setPrototypeOf(subClass, superClass);
}

function _getPrototypeOf(o) {
  _getPrototypeOf = Object.setPrototypeOf ? Object.getPrototypeOf : function _getPrototypeOf(o) {
    return o.__proto__ || Object.getPrototypeOf(o);
  };
  return _getPrototypeOf(o);
}

function _setPrototypeOf(o, p) {
  _setPrototypeOf = Object.setPrototypeOf || function _setPrototypeOf(o, p) {
    o.__proto__ = p;
    return o;
  };

  return _setPrototypeOf(o, p);
}

function _assertThisInitialized(self) {
  if (self === void 0) {
    throw new ReferenceError("this hasn't been initialised - super() hasn't been called");
  }

  return self;
}

function _possibleConstructorReturn(self, call) {
  if (call && (typeof call === "object" || typeof call === "function")) {
    return call;
  }

  return _assertThisInitialized(self);
}

function _slicedToArray(arr, i) {
  return _arrayWithHoles(arr) || _iterableToArrayLimit(arr, i) || _nonIterableRest();
}

function _arrayWithHoles(arr) {
  if (Array.isArray(arr)) return arr;
}

function _iterableToArrayLimit(arr, i) {
  var _arr = [];
  var _n = true;
  var _d = false;
  var _e = undefined;

  try {
    for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) {
      _arr.push(_s.value);

      if (i && _arr.length === i) break;
    }
  } catch (err) {
    _d = true;
    _e = err;
  } finally {
    try {
      if (!_n && _i["return"] != null) _i["return"]();
    } finally {
      if (_d) throw _e;
    }
  }

  return _arr;
}

function _nonIterableRest() {
  throw new TypeError("Invalid attempt to destructure non-iterable instance");
}

var store = createStore({});

var store$1 = createStore({});

function historyUpdater(url, cb) {
  var location = typeof url === 'function' ? url(store$1.state) : url;
  location = location.replace(window.location.origin, '');
  store$1.hydrate({
    location: location
  })(function () {
    cb(location);
  });
}

var history = {
  get state() {
    return store$1.state;
  },

  replaceState: function replaceState(url) {
    historyUpdater(url, function (sanitized) {
      window.history.replaceState({}, '', sanitized);
    });
  },
  pushState: function pushState(url, popstate) {
    historyUpdater(url, function (sanitized) {
      !popstate && window.history.pushState({}, '', sanitized);
    });
  }
};

var Router =
/*#__PURE__*/
function (_React$Component) {
  _inherits(Router, _React$Component);

  function Router(props) {
    var _this;

    _classCallCheck(this, Router);

    _this = _possibleConstructorReturn(this, _getPrototypeOf(Router).call(this, props));
    _this.state = {
      children: props.children.pop ? props.children[0] : props.children
    };
    _this.currentLocation = props.location;
    store$1.listen(function (_ref) {
      var location = _ref.location;

      var _props$router = props.router(location),
          _props$router2 = _slicedToArray(_props$router, 2),
          route = _props$router2[0],
          params = _props$router2[1];

      if (location === _this.currentLocation) return;
      if (route.redirect) return history.pushState(route.redirect.to);
      props.resolve({
        location: location,
        params: params,
        route: route
      }, function (children) {
        _this.currentLocation = location;

        _this.setState({
          children: children
        });
      });
    });
    return _this;
  }

  _createClass(Router, [{
    key: "componentDidMount",
    value: function componentDidMount() {
      window.addEventListener('popstate', function (e) {
        if (!e.target.window) return;
        history.pushState(e.target.location.href, true);
      });
    }
  }, {
    key: "render",
    value: function render() {
      var Child = this.state.children;
      return typeof Child === 'function' ? React.createElement(Child, store.state) : Child;
    }
  }]);

  return Router;
}(React.Component);

/**
 * this is used internally for server.js, client.js, and App.js
 */

function Hypr(_ref) {
  var store = _ref.store,
      router = _ref.router,
      location = _ref.location,
      resolve = _ref.resolve,
      children = _ref.children;
  return React.createElement(react.Provider, {
    store: store
  }, React.createElement(Router, {
    router: router,
    location: location,
    resolve: resolve
  }, children));
}

/**
 * this is passed to rolaStatic as a plugin
 */

function HyprApp(_ref) {
  var _ref$state = _ref.state,
      state = _ref$state === void 0 ? {} : _ref$state,
      pathname = _ref.pathname,
      children = _ref.children;

  /**
   * mock router
   */
  var initialState = Object.assign({}, state, {
    router: {
      location: pathname,
      params: {}
    }
  });
  return React.createElement(Hypr, {
    store: createStore(initialState),
    router: matcher([]),
    location: pathname
  }, children);
}

module.exports = HyprApp;
