'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

function _interopDefault (ex) { return (ex && (typeof ex === 'object') && 'default' in ex) ? ex['default'] : ex; }

var React = _interopDefault(require('react'));
var ReactDOM = _interopDefault(require('react-dom'));
var createStore = _interopDefault(require('picostate'));
var esm = require('flatted/esm');
var matchit = require('matchit');
var react = require('@picostate/react');
var plugins = _interopDefault(require('@/rola.plugins.js'));
var ReactDOMServer = _interopDefault(require('react-dom/server'));
var cx = _interopDefault(require('nanoclass'));

function _classCallCheck(instance, Constructor) {
  if (!(instance instanceof Constructor)) {
    throw new TypeError("Cannot call a class as a function");
  }
}

function _defineProperties(target, props) {
  for (var i = 0; i < props.length; i++) {
    var descriptor = props[i];
    descriptor.enumerable = descriptor.enumerable || false;
    descriptor.configurable = true;
    if ("value" in descriptor) descriptor.writable = true;
    Object.defineProperty(target, descriptor.key, descriptor);
  }
}

function _createClass(Constructor, protoProps, staticProps) {
  if (protoProps) _defineProperties(Constructor.prototype, protoProps);
  if (staticProps) _defineProperties(Constructor, staticProps);
  return Constructor;
}

function _extends() {
  _extends = Object.assign || function (target) {
    for (var i = 1; i < arguments.length; i++) {
      var source = arguments[i];

      for (var key in source) {
        if (Object.prototype.hasOwnProperty.call(source, key)) {
          target[key] = source[key];
        }
      }
    }

    return target;
  };

  return _extends.apply(this, arguments);
}

function _inherits(subClass, superClass) {
  if (typeof superClass !== "function" && superClass !== null) {
    throw new TypeError("Super expression must either be null or a function");
  }

  subClass.prototype = Object.create(superClass && superClass.prototype, {
    constructor: {
      value: subClass,
      writable: true,
      configurable: true
    }
  });
  if (superClass) _setPrototypeOf(subClass, superClass);
}

function _getPrototypeOf(o) {
  _getPrototypeOf = Object.setPrototypeOf ? Object.getPrototypeOf : function _getPrototypeOf(o) {
    return o.__proto__ || Object.getPrototypeOf(o);
  };
  return _getPrototypeOf(o);
}

function _setPrototypeOf(o, p) {
  _setPrototypeOf = Object.setPrototypeOf || function _setPrototypeOf(o, p) {
    o.__proto__ = p;
    return o;
  };

  return _setPrototypeOf(o, p);
}

function _objectWithoutPropertiesLoose(source, excluded) {
  if (source == null) return {};
  var target = {};
  var sourceKeys = Object.keys(source);
  var key, i;

  for (i = 0; i < sourceKeys.length; i++) {
    key = sourceKeys[i];
    if (excluded.indexOf(key) >= 0) continue;
    target[key] = source[key];
  }

  return target;
}

function _objectWithoutProperties(source, excluded) {
  if (source == null) return {};

  var target = _objectWithoutPropertiesLoose(source, excluded);

  var key, i;

  if (Object.getOwnPropertySymbols) {
    var sourceSymbolKeys = Object.getOwnPropertySymbols(source);

    for (i = 0; i < sourceSymbolKeys.length; i++) {
      key = sourceSymbolKeys[i];
      if (excluded.indexOf(key) >= 0) continue;
      if (!Object.prototype.propertyIsEnumerable.call(source, key)) continue;
      target[key] = source[key];
    }
  }

  return target;
}

function _assertThisInitialized(self) {
  if (self === void 0) {
    throw new ReferenceError("this hasn't been initialised - super() hasn't been called");
  }

  return self;
}

function _possibleConstructorReturn(self, call) {
  if (call && (typeof call === "object" || typeof call === "function")) {
    return call;
  }

  return _assertThisInitialized(self);
}

function _slicedToArray(arr, i) {
  return _arrayWithHoles(arr) || _iterableToArrayLimit(arr, i) || _nonIterableRest();
}

function _arrayWithHoles(arr) {
  if (Array.isArray(arr)) return arr;
}

function _iterableToArrayLimit(arr, i) {
  var _arr = [];
  var _n = true;
  var _d = false;
  var _e = undefined;

  try {
    for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) {
      _arr.push(_s.value);

      if (i && _arr.length === i) break;
    }
  } catch (err) {
    _d = true;
    _e = err;
  } finally {
    try {
      if (!_n && _i["return"] != null) _i["return"]();
    } finally {
      if (_d) throw _e;
    }
  }

  return _arr;
}

function _nonIterableRest() {
  throw new TypeError("Invalid attempt to destructure non-iterable instance");
}

function matcher(routes) {
  var matchers = routes.map(function (route) {
    return matchit.parse(route[0]);
  });
  var dictionary = routes.reduce(function (dict, route) {
    dict[route[0]] = route[1];
    return dict;
  }, {});
  return function route(path) {
    var m = matchit.match(path, matchers);
    if (!m.length) return [];
    return [dictionary[m[0].old], matchit.exec(path, m)];
  };
}

var store = createStore({});

function historyUpdater(url, cb) {
  var location = typeof url === 'function' ? url(store.state) : url;
  location = location.replace(window.location.origin, '');
  store.hydrate({
    location: location
  })(function () {
    cb(location);
  });
}

var history = {
  get state() {
    return store.state;
  },

  replaceState: function replaceState(url) {
    historyUpdater(url, function (sanitized) {
      window.history.replaceState({}, '', sanitized);
    });
  },
  pushState: function pushState(url, popstate) {
    historyUpdater(url, function (sanitized) {
      !popstate && window.history.pushState({}, '', sanitized);
    });
  }
};
function withHistory(Component) {
  return function (props) {
    return React.createElement(Component, _extends({
      history: history
    }, props));
  };
}

var store$1 = createStore({});
var withState = react.connect;

var Router =
/*#__PURE__*/
function (_React$Component) {
  _inherits(Router, _React$Component);

  function Router(props) {
    var _this;

    _classCallCheck(this, Router);

    _this = _possibleConstructorReturn(this, _getPrototypeOf(Router).call(this, props));
    _this.state = {
      children: props.children.pop ? props.children[0] : props.children
    };
    _this.currentLocation = props.location;
    store.listen(function (_ref) {
      var location = _ref.location;

      var _props$router = props.router(location),
          _props$router2 = _slicedToArray(_props$router, 2),
          route = _props$router2[0],
          params = _props$router2[1];

      if (location === _this.currentLocation) return;
      if (route.redirect) return history.pushState(route.redirect.to);
      props.resolve({
        location: location,
        params: params,
        route: route
      }, function (children) {
        _this.currentLocation = location;

        _this.setState({
          children: children
        });
      });
    });
    return _this;
  }

  _createClass(Router, [{
    key: "componentDidMount",
    value: function componentDidMount() {
      window.addEventListener('popstate', function (e) {
        if (!e.target.window) return;
        history.pushState(e.target.location.href, true);
      });
    }
  }, {
    key: "render",
    value: function render() {
      var Child = this.state.children;
      return typeof Child === 'function' ? React.createElement(Child, store$1.state) : Child;
    }
  }]);

  return Router;
}(React.Component);

/**
 * this is used internally for server.js, client.js, and App.js
 */

function Hypr(_ref) {
  var store = _ref.store,
      router = _ref.router,
      location = _ref.location,
      resolve = _ref.resolve,
      children = _ref.children;
  return React.createElement(react.Provider, {
    store: store
  }, React.createElement(Router, {
    router: router,
    location: location,
    resolve: resolve
  }, children));
}

function client(routes) {
  var initialState = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
  var options = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {};
  var location = window.location.href.replace(window.location.origin, '');
  var router = matcher(routes.map(function (_ref) {
    var pathname = _ref.pathname,
        route = _objectWithoutProperties(_ref, ["pathname"]);

    return [pathname, route];
  }));

  var _router = router(location),
      _router2 = _slicedToArray(_router, 2),
      route = _router2[0],
      params = _router2[1];

  var serverContext = esm.parse(JSON.stringify(window.__rola));
  store$1.hydrate(Object.assign({}, serverContext.state, initialState, {
    router: {
      location: location,
      params: params
    }
  }));
  var view = route.view;
  var context = {
    state: store$1.state,
    pathname: window.location.pathname
  };
  (plugins || []).filter(function (p) {
    return p.createRoot;
  }).map(function (p) {
    view = p.createRoot({
      app: view(context),
      context: context
    });
  });
  return function render(root) {
    ReactDOM.hydrate(React.createElement(Hypr, {
      store: store$1,
      router: router,
      location: location,
      resolve: function resolve(_ref2, done) {
        var location = _ref2.location,
            params = _ref2.params,
            route = _ref2.route;
        store$1.hydrate({
          router: {
            location: location,
            params: params
          }
        });
        var _route$load = route.load,
            load = _route$load === void 0 ? function () {} : _route$load,
            view = route.view;
        Promise.resolve(load(store$1.state)).then(function (_ref3) {
          var redirect = _ref3.redirect,
              state = _ref3.state;
          if (redirect) return history.pushState(redirect.to);
          var meta = state.meta || {};
          store$1.hydrate(state);
          Promise.resolve(options.resolve ? options.resolve(store$1.state) : null).then(function () {
            document.title = meta.title || document.title;
            var view = route.view;
            var context = {
              state: store$1.state,
              pathname: route.pathname
            };
            (plugins || []).filter(function (p) {
              return p.createRoot;
            }).map(function (p) {
              view = p.createRoot({
                app: view(context),
                context: context
              });
            });
            done(view(context));
          }).catch(function (e) {
            console.error('options.resolve failed', e);
          });
        }).catch(function (e) {
          console.error('route.load failed', e.message || e);
        });
      }
    }, view(context)), root);
  };
}

var createDocument = require('@rola/util/createDocument.js');

var doc = require('@rola/util/document.js');

function redir(res, Location, Referer) {
  var status = arguments.length > 3 && arguments[3] !== undefined ? arguments[3] : 302;
  res.writeHead(status, {
    Location: Location,
    Referer: Referer
  });
  res.end();
}

function server(routes) {
  var initialState = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};
  var router = matcher(routes.map(function (_ref) {
    var pathname = _ref.pathname,
        route = _objectWithoutProperties(_ref, ["pathname"]);

    return [pathname, route];
  }));
  return function handler(req, res, next) {
    var store = createStore({});

    var _router = router(req.url),
        _router2 = _slicedToArray(_router, 2),
        route = _router2[0],
        params = _router2[1];

    if (!route) {
      return next();
    }

    var _route$state = route.state,
        initialRouteState = _route$state === void 0 ? {} : _route$state,
        _route$load = route.load,
        load = _route$load === void 0 ? function () {} : _route$load,
        view = route.view,
        redirect = route.redirect;
    if (redirect) return redir(res, redirect);
    store.hydrate(Object.assign(initialState, initialRouteState, {
      router: {
        location: req.url,
        params: params
      }
    }));
    return Promise.resolve(load(store.state, req)).then(function () {
      var _ref2 = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : {},
          redirect = _ref2.redirect,
          cache = _ref2.cache,
          status = _ref2.status,
          pathname = _ref2.pathname,
          _ref2$state = _ref2.state,
          state = _ref2$state === void 0 ? {} : _ref2$state;

      if (redirect) return redir(res, redirect);
      res.statusCode = status || 200;
      res.setHeader('Content-Type', 'text/html');
      res.setHeader('Cache-Control', typeof cache === 'string' ? cache : cache === false ? "no-cache, no-store, must-revalidate" : "public, max-age=".concat(cache || 86400));
      store.hydrate(state);
      var context = {
        state: store.state,
        pathname: route.pathname || pathname
      };
      (plugins || []).filter(function (p) {
        return p.createRoot;
      }).map(function (p) {
        view = p.createRoot({
          app: view(context),
          context: context
        });
      });
      var tags = createDocument({
        context: context,
        handlers: plugins.filter(function (p) {
          return p.createDocument;
        }).map(function (p) {
          return p.createDocument;
        })
      });
      res.end(doc(Object.assign(tags, {
        context: context,
        view: ReactDOMServer.renderToString(React.createElement(Hypr, {
          store: store,
          router: router,
          location: req.url
        }, view(context)))
      })));
    }).catch(function (e) {
      res.statusCode = 500;
      res.setHeader('Content-Type', 'text/plain');
      res.end('rola error');
      console.log(e);
    });
  };
}

function Link(props) {
  var children = props.children,
      href = props.href,
      className = props.className,
      target = props.target,
      download = props.download,
      rest = _objectWithoutProperties(props, ["children", "href", "className", "target", "download"]);

  var cn = cx([className, history.location === href && 'active']);
  var opts = {};
  if (target) opts.target = target;
  if (download) opts.download = download;
  return React.createElement("a", _extends({
    href: href,
    className: cn,
    onClick: function onClick(e) {
      if (e.ctrlKey || e.metaKey || e.altKey || e.shiftKey || e.defaultPrevented || opts.target === '_blank' || opts.download || /mailto|tel/.test(href) || /^(https?:)?\/\//.test(href)) return;
      e.preventDefault();
      history.pushState(href);
    }
  }, opts, rest), children);
}

var Router$1 =
/*#__PURE__*/
function (_React$Component) {
  _inherits(Router, _React$Component);

  function Router(props) {
    var _this;

    _classCallCheck(this, Router);

    _this = _possibleConstructorReturn(this, _getPrototypeOf(Router).call(this, props));
    _this.state = {
      children: props.children.pop ? props.children[0] : props.children
    };
    _this.currentLocation = props.location;
    store.listen(function (_ref) {
      var location = _ref.location;

      var _props$router = props.router(location),
          _props$router2 = _slicedToArray(_props$router, 2),
          route = _props$router2[0],
          params = _props$router2[1];

      if (location === _this.currentLocation) return;
      if (route.redirect) return history.pushState(route.redirect.to);
      props.resolve({
        location: location,
        params: params,
        route: route
      }, function (children) {
        _this.currentLocation = location;

        _this.setState({
          children: children
        });
      });
    });
    return _this;
  }

  _createClass(Router, [{
    key: "componentDidMount",
    value: function componentDidMount() {
      window.addEventListener('popstate', function (e) {
        if (!e.target.window) return;
        history.pushState(e.target.location.href, true);
      });
    }
  }, {
    key: "render",
    value: function render() {
      var Child = this.state.children;
      return typeof Child === 'function' ? React.createElement(Child, store$1.state) : Child;
    }
  }]);

  return Router;
}(React.Component);

exports.Link = Link;
exports.Router = Router$1;
exports.client = client;
exports.history = history;
exports.server = server;
exports.store = store$1;
exports.withHistory = withHistory;
exports.withState = withState;
